ALTER SESSION SET CURRENT_SCHEMA = TFGADMIN;

/*
DECLARE
   TYPE arrayType IS varray(24) OF VARCHAR2(25);
   table_name arrayType := arrayType (
     'App', 'AppBusinessArea', 'AppGeographicScope', 'ApprovalStatus', 'BusinessArea',
     'Category', 'Change', 'ChangeActionPlan', 'ChangeActionPlanDetail', 'ChangeApproval',
     'ChangeAudit', 'ChangeAuditDetail', 'Criticality', 'Department', 'DepartmentArea',
     'ExecutionResult', 'GCAB', 'GeographicZone', 'Infrastructure', 'Person', 'Rol',
     'Substitution', 'SubstitutionReason', 'Technology'
   );
BEGIN
  FOR i IN 1..table_name.COUNT
  LOOP
    BEGIN
       -- dbms_output.put_line(table_name(i));
       EXECUTE IMMEDIATE 'DROP TABLE ' || table_name(i) || ' CASCADE CONSTRAINTS PURGE';
    EXCEPTION
       WHEN OTHERS THEN
          -- dbms_output.put_line('error:' || table_name(i));
          IF SQLCODE != -942 THEN
             RAISE;
          END IF;
    END;
  END LOOP;
END;
*/


/*
 * Category
 */

CREATE TABLE Category (
  categoryID NUMBER(2) GENERATED BY DEFAULT ON NULL AS IDENTITY START WITH 4 PRIMARY KEY,
  description VARCHAR2(128) NOT NULL,
  created DATE DEFAULT SYSDATE,
  valuation DATE,
  acceptance CHAR(1) CHECK (acceptance IN ('Y', 'N'))
);


/*
 * Criticality
 */

CREATE TABLE Criticality (
  criticalityID NUMBER(1) GENERATED BY DEFAULT ON NULL AS IDENTITY PRIMARY KEY,
  description VARCHAR2(128) NOT NULL,
  criticalityLevel NUMBER(1)
);


/*
 * Department
 */

CREATE TABLE Department (
  departmentID NUMBER(2) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  name VARCHAR2(128) NOT NULL
);


/*
 * DepartmentArea
 */

CREATE TABLE DepartmentArea (
  departmentAreaID NUMBER(2) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  departmentID NUMBER(2) NOT NULL REFERENCES Department(departmentID),
  name VARCHAR2(128) NOT NULL UNIQUE
);


/*
 * GeographicZone
 */

CREATE TABLE GeographicZone (
  geographicZoneID NUMBER(3) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  name VARCHAR2(64) NOT NULL, 
  alpha2 CHAR(2), 
  alpha3 CHAR(3), 
  countryCode CHAR(3), 
  ISO_3166_2 VARCHAR2(13), 
  region VARCHAR2(8), 
  sub_region VARCHAR2(31), 
  intermediate_region VARCHAR2(15), 
  region_code CHAR(3), 
  sub_region_code CHAR(3), 
  intermediate_region_code CHAR(3)
);


/*
 * Infrastructure
 */

CREATE TABLE Infrastructure (
  infrastructureID NUMBER(2) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  description VARCHAR2(128) NOT NULL
);


/*
 * ExecutionResult
 */

CREATE TABLE ExecutionResult (
  executionResultID NUMBER(1) PRIMARY KEY,
  description VARCHAR2(128) NOT NULL,
  actionPlan CHAR(1) NOT NULL CHECK (actionPlan IN ('Y', 'N'))
);


/*
 * Rol
 */

CREATE TABLE Rol (
  rolID NUMBER(1) PRIMARY KEY, --GENERATED BY DEFAULT ON NULL AS IDENTITY START WITH 4
  name VARCHAR2(128)
  --isTechnicalManager CHAR(1) DEFAULT 'N' CHECK (isTechnicalManager IN ('Y', 'N')), 
  --isCoordinatorBusinessAreas CHAR(1) DEFAULT 'N' CHECK (isCoordinatorBusinessAreas IN ('Y', 'N')),
  --isChangeManager CHAR(1) DEFAULT 'N' CHECK (isChangeManager IN ('Y', 'N')),
  --isGCAB CHAR(1) DEFAULT 'N' CHECK (isGCAB IN ('Y', 'N')),
  --isApprover CHAR(1) DEFAULT 'Y' CHECK (isApprover IN ('Y', 'N'))
);
 
/*
 * Technology
 */

CREATE TABLE Technology (
  technologyID NUMBER(2) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  description VARCHAR2(128) NOT NULL
);


/*
 * Person
 */

CREATE TABLE Person (
  personID NUMBER(4) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  givenName VARCHAR2(128) NOT NULL,
  familyName VARCHAR2(128) NOT NULL,
  email VARCHAR2(128),
  phone VARCHAR2(12),
  departmentAreaID NUMBER(2) REFERENCES DepartmentArea(departmentAreaID)
);


/*
 * Business Area
 */

CREATE TABLE BusinessArea (
  businessAreaID NUMBER(2) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  name VARCHAR2(128) NOT NULL,
  contactID NUMBER(4) REFERENCES Person(personID)
);


/*
 * GCAB
 */

CREATE TABLE GCAB (
  gcabID NUMBER(2) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  memberID NUMBER(4) NOT NULL REFERENCES Person(personID) UNIQUE,
  isChangeManager CHAR(1) CHECK (isChangeManager IN ('Y', 'N')), 
  isFullMember CHAR(1) CHECK (isFullMember IN ('Y', 'N'))
);

COMMENT ON TABLE GCAB IS 'Global Change Advisory Board';


/*
 * Substitution
 */

CREATE TABLE Substitution (
  substitutionID NUMBER(3) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  rolID NUMBER(1) REFERENCES Rol(rolID),
  substituteID NUMBER(4) REFERENCES Person(personID),
  position NUMBER(3) DEFAULT 100
  --UNIQUE (rolID, substituteID)
);


/*
 * SubstitutionReason
 */

CREATE TABLE SubstitutionReason (
  substitutionReasonID NUMBER(2) GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
  reason VARCHAR2(128),
  planned CHAR(1) NOT NULL CHECK (planned IN ('Y', 'N'))
);


/*
 * App
 */

CREATE TABLE App (
  appID NUMBER(3) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  name VARCHAR2(128) NOT NULL,
  description VARCHAR2(256),
  toProduction DATE,
  technicalManagerID NUMBER(4) REFERENCES Person(personID),
  coordinatorBusinessAreasID NUMBER(4) REFERENCES Person(personID),    
  averageNumberOfUsers NUMBER(5),
  technologyID NUMBER(2) REFERENCES Technology(technologyID),
  infrastructureID NUMBER(2) REFERENCES Infrastructure(infrastructureID),
  criticalityID NUMBER(2) REFERENCES Criticality(criticalityID),
  economicIimpactWhenNotAvailable NUMBER(8)
);


/*
 * AppBusinessArea
 */

CREATE TABLE AppBusinessArea (
  appID NUMBER(3) NOT NULL REFERENCES App(appID),
  businessAreaID NUMBER(2) NOT NULL REFERENCES BusinessArea(businessAreaID)
);


/*
 * AppGeographicScope
 */

CREATE TABLE AppGeographicScope (
  appID NUMBER(3) NOT NULL REFERENCES App(appID),
  geographicZoneID NUMBER(3) NOT NULL REFERENCES GeographicZone(geographicZoneID)
);


/*
 * ApprovalStatus
 */

CREATE TABLE ApprovalStatus (
  approvalStatusID NUMBER(1) PRIMARY KEY,
  status VARCHAR2(128) NOT NULL
);


/*
 * Change
 */

CREATE TABLE Change (
  changeID NUMBER(4) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  appID NUMBER(3) REFERENCES App(appID),
  changeDate DATE DEFAULT SYSDATE,
  version VARCHAR2(16),
  comments VARCHAR2(256),
  categoryID NUMBER(1) REFERENCES Category(categoryID),
  approvalStatusID NUMBER(1) DEFAULT 0 REFERENCES ApprovalStatus(approvalStatusID),
  approvalStatusTimestamp DATE,
  executionResultID NUMBER(1) REFERENCES ExecutionResult(executionResultID),
  executionResultTimestamp DATE
);


/*
 * ChangeGeographicScope
 */

CREATE TABLE ChangeGeographicScope (
  changeID NUMBER(3) NOT NULL REFERENCES Change(changeID),
  geographicZoneID NUMBER(3) NOT NULL REFERENCES GeographicZone(geographicZoneID)
);


/*
 * ChangeApprover
 */

CREATE TABLE ChangeApprover (
  changeID NUMBER(4) REFERENCES Change(changeID),
  rolID NUMBER(2) NOT NULL REFERENCES Rol(rolID),
  usualID NUMBER(4) NOT NULL REFERENCES Person(personID),
  substituteID NUMBER(4) REFERENCES Person(personID),
  substitutionReasonID NUMBER(1) REFERENCES SubstitutionReason(substitutionReasonID),
  approved CHAR(1) CHECK (approved IN ('Y', 'N')),
  approvedTimestamp DATE, --en Oracle, el tipo de datos DATE almacena fecha y hora
  comments VARCHAR2(1024),
  PRIMARY KEY (changeID, usualID)
);


/*
 * ChangeApproval
 */
/*
CREATE TABLE ChangeApproval (
  changeID NUMBER(4) PRIMARY KEY REFERENCES Change(changeID),
  toApproval NUMBER(2) DEFAULT 0,
  approved NUMBER(2) DEFAULT 0,
  disapproved NUMBER(2) DEFAULT 0
);
*/

/*
 * ChangeActionPlan
 */

CREATE TABLE ChangeActionPlan (
  actionPlanID NUMBER(4) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  changeID NUMBER(4) REFERENCES Change(changeID),
  opened DATE DEFAULT SYSDATE,
  closing DATE DEFAULT SYSDATE + 7,  --fecha inicialmente definida para cerrar el plan de acción
  responsibleID NUMBER(4) REFERENCES Person(personID),
  closed DATE,  --fecha real de cierre del plan de acción
  closedPersonID NUMBER(4) REFERENCES Person(personID)
);


/*
 * ChangeActionPlanLine
 */

CREATE TABLE ChangeActionPlanLine (
  actionPlanLineID NUMBER(5) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  actionPlanID NUMBER(4) REFERENCES ChangeActionPlan(ActionPlanID),
  description VARCHAR2(256) NOT NULL,
  closed DATE
);


/*
 * ChangeAudit
 */

CREATE TABLE ChangeAudit (
  auditID NUMBER(4) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  changeID NUMBER(4) REFERENCES Change(ChangeID),
  auditStart DATE DEFAULT SYSDATE,
  auditEnd DATE
);


/*
 * ChangeAuditDetail
 */

CREATE TABLE ChangeAuditDetail (
  auditDetailID NUMBER(5) GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  auditID NUMBER(4) REFERENCES ChangeAudit(auditID),
  description VARCHAR2(256) NOT NULL,
  nonCompliance CHAR(1) DEFAULT 'N' CHECK (nonCompliance IN ('Y', 'N'))
);
