ALTER SESSION SET CURRENT_SCHEMA = TFGADMIN;

--ALTER SESSION SET NLS_DATE_FORMAT = 'DD-MM-YYYY HH24:MI:SS'

/*
 * Repositorio estadístico
 */


/*
 * Borrar todas las tablas del repositorio estadístico
*/

/*
BEGIN
  FOR i IN (SELECT table_name FROM all_tables WHERE table_name LIKE 'STATISTIC%' AND owner = 'TFGADMIN') LOOP
    EXECUTE IMMEDIATE 'DROP TABLE ' || i.TABLE_NAME || ' CASCADE CONSTRAINTS PURGE';
  END LOOP;
END;
*/


/*
 * Table StatisticChangesInApprovalProcess
 * En el momento de ejecutar la consulta, número de cambios registrados en el sistema que están en proceso de aprobación
 */

CREATE TABLE StatisticChangesInApprovalProcess (
  numChangesInApprovalProcess NUMBER(4) DEFAULT 0
);
INSERT INTO StatisticChangesInApprovalProcess (numChangesInApprovalProcess) VALUES (default);

/*
 * Table StatisticNotSucceedExecutionsLastYear
 * En el último año, número de cambios aprobados cuya ejecución no haya sido correcta
 */

CREATE TABLE StatisticNotSucceedExecutionsLastYear (
  currentYear CHAR(4) DEFAULT EXTRACT(YEAR FROM SYSDATE),
  numNotSucceedExecutionsCurrentYear NUMBER(4) DEFAULT 0,
  numNotSucceedExecutionsPreviousYear NUMBER(4) DEFAULT 0
);
INSERT INTO StatisticNotSucceedExecutionsLastYear (currentYear) VALUES (default);


/* 
 * Table StatisticTopTechnicalManagerCorrectExecutions
 * Teniendo en cuenta toda la información existente en la BD, responsable técnico con más cambios ejecutados de manera correcta
 */

CREATE TABLE StatisticTopTechnicalManagerCorrectExecutions (
  technicalManagerID NUMBER(4),
  numCorrectExecutions NUMBER(4)
);
CREATE UNIQUE INDEX idx_StatisticTopTechnicalManagerCorrectExecutions ON StatisticTopTechnicalManagerCorrectExecutions(technicalManagerID);

/*
 * Table StatisticClosedActionPlanLinesOnTimePercentageYear
 * Dado un año concreto, porcentaje de acciones definidas para ejecuciones no correctas cerradas en el tiempo inicialmente definido en el plan de acción correspondiente
 */

CREATE TABLE StatisticClosedActionPlanLinesOnTimePercentageYear (
  year CHAR(4) DEFAULT EXTRACT(YEAR FROM SYSDATE),
  totalActionPlanLines NUMBER(3) DEFAULT 0,
  onTimeActionPlanLines NUMBER(3) DEFAULT 0
);
CREATE UNIQUE INDEX idx_StatisticClosedActionPlanLinesOnTimePercentageYear ON StatisticClosedActionPlanLinesOnTimePercentageYear(year);


/*
 * Table StatisticGCABApprovalsCurrentYear
 * En el año en curso, número total de cambios aprobados en la GCAB
 */

CREATE TABLE StatisticGCABApprovalsCurrentYear (
  currentYear CHAR(4) DEFAULT EXTRACT(YEAR FROM SYSDATE),
  numGCABApprovalsCurrentYear NUMBER(4) DEFAULT 0
);
INSERT INTO StatisticGCABApprovalsCurrentYear (currentYear) VALUES (default);


/*
 * Table StatisticDisapprovedChangesYear
 * En un año concreto, número de cambios que no fueron aprobados y, consecuentemente, no se ejecutaron
 */

CREATE TABLE StatisticDisapprovedChangesYear (
  year CHAR(4),
  numDisapprovedChanges NUMBER(4)
);
CREATE UNIQUE INDEX idx_StatisticDisapprovedChangesYear ON StatisticDisapprovedChangesYear(year);


/*
 * Table StatisticReplannedChangesPercentageRegion
 * Dada una región geográfica concreta, porcentaje de cambios que su ejecución se replanificó
 */

CREATE TABLE StatisticReplannedChangesPercentageRegion (
  geographicZoneID NUMBER(3),
  totalExecution NUMBER(4),
  replannedExecution NUMBER(4)
);
CREATE UNIQUE INDEX idx_StatisticReplannedChangesPercentageRegion ON StatisticReplannedChangesPercentageRegion(geographicZoneID);


/*
 * Table StatisticNonCompliancesDetectedLastSixMonths
 *
 * En los últimos 6 meses, número total de incumplimientos detectados durante las auditorías realizadas
 */

CREATE TABLE StatisticNonCompliancesSixMonths (
  lastMonth CHAR(6) DEFAULT TO_CHAR(SYSDATE, 'YYYYMM'),
  month1 NUMBER(2) DEFAULT 0,
  month2 NUMBER(2) DEFAULT 0,
  month3 NUMBER(2) DEFAULT 0,
  month4 NUMBER(2) DEFAULT 0,
  month5 NUMBER(2) DEFAULT 0,
  month6 NUMBER(2) DEFAULT 0
);
INSERT INTO StatisticNonCompliancesSixMonths (lastMonth) VALUES (default);


/*
 * Table StatisticOpenedActionPlans
 *
 * En el momento de ejecutar la consulta, número total de planes de acción sobre ejecuciones no correctas que están abiertos
 */

CREATE TABLE StatisticOpenedActionPlans (
  numOpenedActionPlans NUMBER(4)
);
INSERT INTO StatisticOpenedActionPlans (numOpenedActionPlans) VALUES (0);


/*
 * Table StatisticTopTechnicalManagerChangesInProgress
 * En un momento dado, responsable técnico con más cambios en curso
 */

CREATE TABLE StatisticTopTechnicalManagerChangesInProgress (
  technicalManagerID NUMBER(4),
  numChangesInProgress NUMBER(4)
);
CREATE UNIQUE INDEX idx_StatisticTopTechnicalManagerChangesInProgress ON StatisticTopTechnicalManagerChangesInProgress(technicalManagerID);


/*
 * Table StatisticSubstitutionApprover
 * Teniendo en cuenta todos los datos de la BD, persona concreta con rol de aprobador que ha estado sustituida más veces por indisponibilidad no planificada
 */

CREATE TABLE StatisticSubstitutionApprover (
  personID NUMBER(4),
  numSubstitutions NUMBER(4)
);
CREATE UNIQUE INDEX idx_StatisticSubstitutionApprover ON StatisticSubstitutionApprover(personID);
