
-- Borrar todos los procedimientos almacenados
BEGIN
  FOR i IN (SELECT object_name FROM ALL_PROCEDURES WHERE owner = 'TFGADMIN' AND object_name NOT LIKE 'BIN%')
  LOOP
    EXECUTE IMMEDIATE 'DROP PROCEDURE ' || i.OBJECT_NAME;
  END LOOP;
END;


/*
 * Procedimiento almacenado con MERGE para insertar o actualizar en la tabla Person
 * Si se pasa un ID que no existe lo daría de alta, no lo comprueba previamente
 */

/*

CREATE OR REPLACE PROCEDURE merge_Person (
  p_personID Person.personID%TYPE,
  p_givenName Person.givenName%TYPE,
  p_familyName Person.familyName%TYPE,
  p_email Person.email%TYPE,
  p_phone Person.phone%TYPE,
  p_departmentAreaID Person.departmentAreaID%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    MERGE INTO Person p
    USING (SELECT p_personID AS personID FROM dual) b
    ON (p.personID = b.personID)
    WHEN MATCHED THEN
      UPDATE SET p.givenName = p_givenName,
                 p.familyName = p_familyName,
                 p.email = p_email,
                 p.phone = p_phone,
                 p.departmentAreaID = p_departmentAreaID
    WHEN NOT MATCHED THEN
      INSERT (givenName, familyName, email, phone, departmentAreaID)
      VALUES (p_givenName, p_familyName, p_email, p_phone, p_departmentAreaID);
    
    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error, rollback y retorno
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_personID || '|' || p_givenName || '|' || p_familyName || '|' || p_email || '|' || p_phone || '|' || p_departmentAreaID;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP); 

END merge_Person;
*/


/*
 * Procedimiento almacenado para insertar o actualizar en la tabla Person
 */
 
CREATE OR REPLACE PROCEDURE upsert_Person (
  p_personID IN Person.personID%TYPE,
  p_givenName IN Person.givenName%TYPE,
  p_familyName IN Person.familyName%TYPE,
  p_email IN Person.email%TYPE,
  p_phone IN Person.phone%TYPE,
  p_departmentAreaID IN Person.departmentAreaID%TYPE,
  RSP OUT VARCHAR2
  ) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;
  
  BEGIN
    -- Si el personID es nulo, se hace un insert
    IF p_personID IS NULL THEN
      INSERT INTO Person (givenName, familyName, email, phone, departmentAreaID)
      VALUES (p_givenName, p_familyName, p_email, p_phone, p_departmentAreaID);
    -- Si el personID no es nulo, se hace un update
    ELSE
      UPDATE Person
      SET givenName = p_givenName,
          familyName = p_familyName,
          email = p_email,
          phone = p_phone,
          departmentAreaID = p_departmentAreaID
      WHERE personID = p_personID;
      -- Si el número de filas afectadas es cero, se lanza una excepción
      IF SQL%ROWCOUNT = 0 THEN
        RAISE_APPLICATION_ERROR(-20001, 'No existe una persona con el ID ' || p_personID);
      END IF;
    END IF;

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues :=   p_personID || '|' || p_givenName || '|' || p_familyName || '|' || p_email || '|' || p_phone || '|' || p_departmentAreaID;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);

END upsert_Person;


/*
 * Procedimiento almacenado para eliminar un registro de la tabla Person
 */

CREATE OR REPLACE PROCEDURE delete_Person (
  p_personID IN Person.personID%TYPE,
  RSP OUT VARCHAR2
  ) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;
  
  BEGIN
    -- Se elimina el registro con el personID pasado
    DELETE FROM Person
    WHERE personID = p_personID;
    
    -- Si el número de filas afectadas ha sido cero, se lanza una excepción
    IF SQL%ROWCOUNT = 0 THEN
      RAISE_APPLICATION_ERROR(-20001, 'No existe una persona con el ID ' || p_personID);
    END IF;
      

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues :=   p_personID;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);

END delete_Person;



/*
 * Procedimiento almacenado con MERGE para insertar o actualizar en la tabla App
 * Si se pasa un ID que no existe lo daría de alta, no lo comprueba previamente
 */

/*

CREATE OR REPLACE PROCEDURE merge_App(
  p_appID App.appID%TYPE,
  p_name App.name%TYPE,
  p_description App.description%TYPE,
  p_toProduction App.toProduction%TYPE,
  p_technicalManagerID App.technicalManagerID%TYPE,
  p_coordinatorBusinessAreasID App.coordinatorBusinessAreasID%TYPE,
  p_averageNumberOfUsers App.averageNumberOfUsers%TYPE,
  p_technologyID App.technologyID%TYPE,
  p_infrastructureID App.infrastructureID%TYPE,
  p_criticalityID App.criticalityID%TYPE,
  p_economicIimpactWhenNotAvailable App.economicIimpactWhenNotAvailable%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    MERGE INTO App a
    USING (SELECT p_appID AS appID FROM dual) b
    ON (a.appID = b.appID)
    WHEN MATCHED THEN
      UPDATE SET a.name = p_name,
                 a.description = p_description,
                 a.toProduction = p_toProduction,
                 a.technicalManagerID = p_technicalManagerID,
                 a.coordinatorBusinessAreasID = p_coordinatorBusinessAreasID,
                 a.averageNumberOfUsers = p_averageNumberOfUsers,
                 a.technologyID = p_technologyID,
                 a.infrastructureID = p_infrastructureID,
                 a.criticalityID = p_criticalityID,
                 a.economicIimpactWhenNotAvailable = p_economicIimpactWhenNotAvailable
    WHEN NOT MATCHED THEN
      INSERT (name, description, toProduction, technicalManagerID, coordinatorBusinessAreasID, averageNumberOfUsers, technologyID, infrastructureID, criticalityID, economicIimpactWhenNotAvailable)
      VALUES (p_name, p_description, p_toProduction, p_technicalManagerID, p_coordinatorBusinessAreasID, p_averageNumberOfUsers, p_technologyID, p_infrastructureID, p_criticalityID, p_economicIimpactWhenNotAvailable);
      
      -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
      COMMIT;
      RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error, rollback y retorno
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_appID || '|' || p_name || '|' || p_description || '|' || p_toProduction || '|' || p_technicalManagerID || '|' || p_coordinatorBusinessAreasID || '|' || p_averageNumberOfUsers || '|' || p_technologyID || '|' || p_infrastructureID || '|' || p_criticalityID || '|' || p_economicIimpactWhenNotAvailable;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);  

END merge_App;
*/

/*
 * Procedimiento almacenado para insertar una aplicación
 */

CREATE OR REPLACE PROCEDURE upsert_App (
  p_appID IN App.appID%TYPE,
  p_name IN App.name%TYPE,
  p_description IN App.description%TYPE,
  p_toProduction IN App.toProduction%TYPE,
  p_technicalManagerID IN App.technicalManagerID%TYPE,
  p_coordinatorBusinessAreasID IN App.coordinatorBusinessAreasID%TYPE,
  p_averageNumberOfUsers IN App.averageNumberOfUsers%TYPE,
  p_technologyID IN App.technologyID%TYPE,
  p_infrastructureID IN App.infrastructureID%TYPE,
  p_criticalityID IN App.criticalityID%TYPE,
  p_economicIimpactWhenNotAvailable IN App.economicIimpactWhenNotAvailable%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;
  
  BEGIN
    IF p_appID IS NULL THEN
      -- Inserta una nueva aplicación
      INSERT INTO App (name, description, toProduction, technicalManagerID, coordinatorBusinessAreasID, averageNumberOfUsers, technologyID, infrastructureID, criticalityID, economicIimpactWhenNotAvailable)
      VALUES (p_name, p_description, p_toProduction, p_technicalManagerID, p_coordinatorBusinessAreasID, p_averageNumberOfUsers, p_technologyID, p_infrastructureID, p_criticalityID, p_economicIimpactWhenNotAvailable);
    
    ELSE
      -- Update the existing App
      UPDATE App
      SET name = p_name,
          description = p_description,
          toProduction = p_toProduction,
          technicalManagerID = p_technicalManagerID,
          coordinatorBusinessAreasID = p_coordinatorBusinessAreasID,
          averageNumberOfUsers = p_averageNumberOfUsers,
          technologyID = p_technologyID,
          infrastructureID = p_infrastructureID,
          criticalityID = p_criticalityID,
          economicIimpactWhenNotAvailable = p_economicIimpactWhenNotAvailable
      WHERE appID = p_appID;

      -- Si el número de filas afectadas es cero, se lanza una excepción
      IF SQL%ROWCOUNT = 0 THEN
        RAISE_APPLICATION_ERROR(-20001, 'No existe una app con el ID ' || p_appID);
      END IF;
    END IF;

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_appID || '|' || p_name || '|' || p_description || '|' || p_toProduction || '|' || p_technicalManagerID || '|' || p_coordinatorBusinessAreasID || '|' || p_averageNumberOfUsers || '|' || p_technologyID || '|' || p_infrastructureID || '|' || p_criticalityID || '|' || p_economicIimpactWhenNotAvailable;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);

END upsert_App;


/*
 * Procedimiento almacenado para eliminar un registro de la tabla Person
 */

CREATE OR REPLACE PROCEDURE delete_App (
  p_appID IN App.appID%TYPE,
  RSP OUT VARCHAR2
  ) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    -- Se elimina el registro con el personID suministrado
    DELETE FROM App
    WHERE appID = p_appID;

    -- Si el número de filas afectadas es cero, se lanza una excepción
    IF SQL%ROWCOUNT = 0 THEN
      RAISE_APPLICATION_ERROR(-20001, 'No existe una aplicación con el ID ' || p_appID);
    END IF;
  
    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_appID ;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);

END delete_App;


/*
 * Procedimiento almacenado para insertar un cambio
 */

CREATE OR REPLACE PROCEDURE insert_Change (
    p_appID IN Change.appID%TYPE,
    p_changeDate IN Change.changeDate%TYPE,
    p_version IN Change.version%TYPE,
    p_comments IN Change.comments%TYPE,
    p_categoryID IN Change.categoryID%TYPE,
    RSP OUT VARCHAR2
)
AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
  v_changeID Change.changeID%TYPE;

BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN

    -- Insertar en la tabla Change
    INSERT INTO Change (appID, changeDate, version, comments, categoryID)
    VALUES (p_appID, p_changeDate, p_version, p_comments, p_categoryID)
    RETURNING changeID INTO v_changeID;

    -- Insertar en la tabla ChangeApproval
    INSERT INTO ChangeApproval (changeID)
    VALUES (v_changeID);

    -- Insertar en la tabla ChangeApprover
    INSERT INTO ChangeApprover (changeID, rolID, usualID)
    VALUES (v_changeID, 1, (SELECT technicalManagerID FROM App WHERE AppID = p_appID));

    INSERT INTO ChangeApprover (changeID, rolID, usualID)
    VALUES (v_changeID, 2, (SELECT coordinatorBusinessAreasID FROM App WHERE AppID = p_appID));

    INSERT INTO ChangeApprover (changeID, rolID, usualID)
    VALUES (v_changeID, 3, (SELECT memberID FROM GCAB WHERE isChangeManager = 'Y'));

    INSERT INTO ChangeApprover (changeID, rolID, usualID)
    SELECT v_changeID, 4, memberID FROM GCAB WHERE isChangeManager = 'N';

    -- Insertar en la tabla ChangeGeographicScope
    INSERT INTO ChangeGeographicScope (changeID, geographicZoneID)
    SELECT v_changeID, geographicZoneID
    FROM AppGeographicScope
    WHERE appID = p_appID;

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_appID || '|' || p_changeDate || '|' || p_version || '|' || SUBSTR(p_comments, 1, 32) || '|' || p_categoryID;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
    
END insert_Change;


/*
 * Procedimiento almacenado para notificar una sustitución
 */

CREATE OR REPLACE PROCEDURE substitute_ChangeApprover (
  p_changeID IN ChangeApprover.changeID%TYPE,
  p_rolID IN ChangeApprover.rolID%TYPE,
  p_substituteID IN ChangeApprover.substituteID%TYPE,
  p_substitutionReasonID IN ChangeApprover.substitutionReasonID%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
  v_substituteID Substitution.substituteID%TYPE;
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    /*
    -- No está definida ninguna causa de ausencia con ese ID
    IF NOT EXISTS(SELECT 1 FROM SubstitutionReason WHERE substitutionReasonID = p_substitutionReasonID) THEN
        RSP := 'No está definida esa causa de ausencia';
        RETURN;
    END IF;
    */
    
    IF p_substituteID IS NULL THEN
      SELECT substituteID INTO v_substituteID FROM Substitution WHERE rolID = p_rolID ORDER BY position FETCH FIRST ROW ONLY;    
    ELSE
      -- Si se pasa el ID del sustituto, comprobar que esté asignado al rolID de interés
      SELECT substituteID INTO v_substituteID FROM Substitution WHERE rolID = p_rolID AND substituteID = p_substituteID;    
    END IF;

    -- Si un sustituto no ha sido localizado, lanza un error, en caso contrario, continúa para actualizar
    UPDATE ChangeApprover
    SET substituteID = v_substituteID, substitutionReasonID = p_substitutionReasonID
    WHERE changeID = p_changeID AND rolID = p_rolID AND EXISTS (SELECT 1 FROM Change WHERE ChangeID = p_changeID AND approvalStatusID = 0);

    IF SQL%ROWCOUNT = 0 THEN
      RAISE_APPLICATION_ERROR(-20001, 'No existe el cambio o está cerrado ' || p_changeID);
    END IF;  

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_changeID || '|' || p_rolID || '|' || p_substitutionReasonID;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
    
END substitute_ChangeApprover;



/*
 * Procedimiento almacenado para registrar una aprobación
 */

CREATE OR REPLACE PROCEDURE approved_ChangeApprover (
  p_changeID IN ChangeApprover.changeID%TYPE,
  p_personID IN ChangeApprover.usualID%TYPE,
  p_approved IN ChangeApprover.approved%TYPE,
  p_approvedTimeStamp IN ChangeApprover.approvedTimeStamp%TYPE,
  p_comments IN ChangeApprover.comments%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
  v_approved CHAR(1);
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    UPDATE ChangeApprover
    SET approved = p_approved, approvedTimeStamp = CASE WHEN p_approvedTimeStamp IS NOT NULL THEN p_approvedTimeStamp ELSE SYSDATE END, comments = p_comments
    WHERE changeID = p_changeID AND (usualID = p_personID OR substituteID = p_personID) AND approved IS NULL;

    IF SQL%ROWCOUNT = 0 THEN
      RAISE_APPLICATION_ERROR(-20001, 'No existe el cambio ' || p_changeID || ' o la persona con ID ' || p_personID || ' ya ha votado');
    END IF;  

    --Ahora vamos a verificar si ya han votado todos
    SELECT
      CASE WHEN EXISTS(SELECT * FROM ChangeApprover WHERE ChangeID = p_changeID AND approved IS NULL) THEN NULL
           WHEN EXISTS(SELECT * FROM ChangeApprover WHERE ChangeID = p_changeID AND approved = 'N') THEN 'N'
           ELSE 'Y' END into v_approved
    FROM DUAL;

    --Si no hay pendientes de voto, actualiza la tabla de cambios, change, con el resultado global
    IF v_approved IS NOT NULL THEN
      UPDATE Change
      SET approvalStatusID = CASE WHEN v_approved = 'Y' THEN 1 ELSE 2 END,
          approvalStatusTimeStamp = SYSDATE
      WHERE changeID = p_changeID;
    END IF;

    -- Si todo ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_changeID || '|' || p_personID || '|' || p_approved || '|' || p_approvedTimestamp || '|' || SUBSTR(p_comments, 1, 32);
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
    
END approved_ChangeApprover;


/*
 * Procedimiento almacenado para registrar el resultado de la ejecución
 */

CREATE OR REPLACE PROCEDURE executionResult_Change (
  p_changeID IN Change.changeID%TYPE,
  p_executionResultID IN Change.executionResultID%TYPE,
  p_executionResultTimeStamp IN Change.executionResultTimeStamp%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN 
    UPDATE Change
    SET executionResultID = p_executionResultID, executionResultTimeStamp = CASE WHEN p_executionResultTimeStamp IS NOT NULL THEN p_executionResultTimeStamp ELSE SYSDATE END
    WHERE changeID = p_changeID AND approvalStatusID = 1;

    IF SQL%ROWCOUNT = 0 THEN
      RAISE_APPLICATION_ERROR(-20001, 'No existe el cambio o su estado no es "Aprobado"');
    END IF;  

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_changeID || '|' || p_executionResultID || '|' || p_executionResultTimeStamp;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
     
END executionResult_Change;


/*
 * Procedimiento almacenado para modificar la fecha prevista de cierre
 */

CREATE OR REPLACE PROCEDURE closing_ChangeActionPlan (
  p_changeID IN ChangeActionPlan.changeID%TYPE,
  p_closing IN ChangeActionPlan.closing%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;
  
  BEGIN
    UPDATE ChangeActionPlan
    SET closing = p_closing
    WHERE changeID = p_changeID AND closed IS NULL;

    IF SQL%ROWCOUNT = 0 THEN
      RAISE_APPLICATION_ERROR(-20001, 'El plan de acción no existe o ya está cerrado');
    END IF;  

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_changeID || '|' || p_closing;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
     
END closing_ChangeActionPlan;


/*
 * Procedimiento almacenado para añadir una acción a un plan de acción activo
 */

CREATE OR REPLACE PROCEDURE insert_ChangeActionPlanLine (
  p_actionPlanID IN ChangeActionPlanLine.actionPlanID%TYPE,
  p_description IN ChangeActionPlanLine.description%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
  v_closed ChangeActionPlan.closed%TYPE;
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    -- Comprobar que el plan de acción no esté cerrado, sino existiese dará error no_data_found
    SELECT closed INTO v_closed FROM ChangeActionPlan WHERE actionPlanID = p_actionPlanID;

    IF v_closed IS NULL THEN
      -- Insert into ChangeActionPlanLine table
      INSERT INTO ChangeActionPlanLine (actionPlanID, description)
      VALUES (p_actionPlanID, p_description);

      -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
      COMMIT;
      RSP := 'OK';

    ELSE
      RAISE_APPLICATION_ERROR(-20001, 'No se puede insertar el registro, el plan de acción está cerrado.');
    END IF;

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_actionPlanID || '|' || p_description;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
     
END insert_ChangeActionPlanLine;



/*
 * Procedimiento almacenado para cerrar un plan de acción abierto
 */

CREATE OR REPLACE PROCEDURE close_ChangeActionPlan (
  p_changeID IN ChangeActionPlan.changeID%TYPE,
  p_closedPersonID ChangeActionPlan.closedPersonID%TYPE,
  p_Closed ChangeActionPlan.closed%TYPE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
  v_closedPersonID NUMBER(4);
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    SELECT CASE WHEN substituteID IS NULL THEN usualID ELSE substituteID END INTO v_closedPersonID
    FROM ChangeApprover WHERE changeID = p_changeID AND rolID = 3;

    IF p_closedPersonID != v_closedPersonID THEN
      RAISE_APPLICATION_ERROR(-20001, 'La persona que valide el cierre debe ser el gestor de cambios');
    END IF;

    UPDATE ChangeActionPlan
    SET closedPersonID = p_closedPersonID, closed = CASE WHEN p_Closed IS NOT NULL THEN p_Closed ELSE SYSDATE END
    WHERE changeID = p_changeID AND closed IS NULL;

    IF SQL%ROWCOUNT = 0 THEN
      RAISE_APPLICATION_ERROR(-20001, 'No existe el cambio o no tiene un plan de acción asociado o ya está cerrado');
    END IF;  

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_changeID || '|' || p_closedPersonID || '|' || p_Closed;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
     
END close_ChangeActionPlan;


/*
 * Procedimiento almacenado para crear una auditoría
 */

CREATE OR REPLACE PROCEDURE insert_ChangeAudit (
  p_changeId IN Change.changeID%TYPE,
  p_auditStart IN ChangeAudit.auditStart%TYPE DEFAULT SYSDATE,
  RSP OUT VARCHAR2
)
AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    INSERT INTO ChangeAudit (changeID, auditStart)
    VALUES (p_changeId, NVL(p_auditStart, SYSDATE));

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_changeID || '|' || p_auditStart;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
     
END insert_ChangeAudit;


/*
 * Procedimiento almacenado para añadir registros de una auditoría abierta
 */

CREATE OR REPLACE PROCEDURE insert_ChangeAuditDetail (
  p_auditID IN ChangeAuditDetail.auditID%TYPE,
  p_description IN ChangeAuditDetail.description%TYPE,
  p_nonCompliance IN ChangeAuditDetail.nonCompliance%TYPE DEFAULT 'Y',
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
  v_auditEnd ChangeAudit.auditEnd%TYPE;
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    -- Comprobar que la auditoría no esté cerrada, sino existiese dará error no_data_found
    SELECT auditEnd INTO v_auditEnd FROM ChangeAudit WHERE auditID = p_auditID;

    IF v_auditEnd IS NULL THEN
      -- Insert into ChangeAuditDetail table
      INSERT INTO ChangeAuditDetail (auditID, description, nonCompliance)
      VALUES (p_auditID, p_description, p_nonCompliance);

      -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
      COMMIT;
      RSP := 'OK';

    ELSE
      RAISE_APPLICATION_ERROR(-20001, 'No se puede insertar el registro, la auditoría está cerrada.');
    END IF;

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_auditID || '|' || p_description || '|' || p_nonCompliance;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
     
END insert_ChangeAuditDetail;


/*
 * Procedimiento almacenado para cerrar una auditoría abierta
 */

CREATE OR REPLACE PROCEDURE end_ChangeAudit (
  p_auditID IN ChangeAudit.auditID%TYPE,
  p_auditEnd IN ChangeAudit.auditEnd%TYPE DEFAULT SYSDATE,
  RSP OUT VARCHAR2
) AS
  v_procedureName VARCHAR2(128);
  v_paramValues VARCHAR2(512);
BEGIN
  v_procedureName := $$PLSQL_UNIT;

  BEGIN
    UPDATE ChangeAudit
    SET auditEnd = NVL(p_auditEnd, SYSDATE)
    WHERE auditID = p_auditID AND auditEnd IS NULL;

    IF SQL%ROWCOUNT = 0 THEN
      RAISE_APPLICATION_ERROR(-20001, 'No se está auditando ese cambio o la auditoría ya finalizó');
    END IF;  

    -- Si ha ido bien, se confirma la transacción y se asigna el valor 'OK' al parámetro de salida
    COMMIT;
    RSP := 'OK';

  EXCEPTION
    -- Se ha producido un error
    WHEN OTHERS THEN
      ROLLBACK;
      -- Se asigna el valor 'ERROR' seguido del mensaje de error
      RSP := 'ERROR ' || SQLERRM;
      v_paramValues := p_auditID || '|' || p_auditEnd;
  END;

  -- Se inserta un registro en la tabla de log con el nombre del procedimiento, el valor de RSP, el instante y el usuario de la BD
  pkg_log.insertLog(v_procedureName, v_paramValues, RSP);
     
END end_ChangeAudit;


