--En el momento de ejecutar la consulta, número de cambios registrados en el sistema que están en proceso de aprobación
UPDATE StatisticChangesInApprovalProcess
SET numChangesInApprovalProcess = (SELECT COUNT(*) FROM Change WHERE approvalStatusID = 0);

--En el último año, número de cambios aprobados cuya ejecución no haya sido correcta
UPDATE StatisticNotSucceedExecutionsLastYear
SET numNotSucceedExecutionsPreviousYear = (SELECT COUNT(*) FROM Change WHERE executionResultID > 1 AND EXTRACT(YEAR FROM changeDate) = EXTRACT(YEAR FROM SYSDATE) - 1),
    numNotSucceedExecutionsCurrentYear  = (SELECT COUNT(*) FROM Change WHERE executionResultID > 1 AND EXTRACT(YEAR FROM changeDate) = EXTRACT(YEAR FROM SYSDATE));

--Teniendo en cuenta toda la información existente en la BD, responsable técnico con más cambios ejecutados de manera correcta
DELETE StatisticTopTechnicalManagerCorrectExecutions;
INSERT INTO StatisticTopTechnicalManagerCorrectExecutions
SELECT CASE WHEN substituteID IS NOT NULL THEN substituteID ELSE usualID END, COUNT(*)
FROM ChangeApprover a JOIN Change c ON a.changeID = c.changeID
WHERE rolID = 1 AND executionResultID = 1
GROUP BY CASE WHEN substituteID IS NOT NULL THEN substituteID ELSE usualID END;

--Dado un año concreto, porcentaje de acciones definidas para ejecuciones no correctas cerradas en el tiempo inicialmente definido en el plan de acción correspondiente
DELETE StatisticClosedActionPlanLinesOnTimePercentageYear;
INSERT INTO StatisticClosedActionPlanLinesOnTimePercentageYear
SELECT EXTRACT(YEAR FROM SYSDATE), COUNT(*), COUNT(CASE WHEN l.closed <= p.closed THEN 1 END)
FROM Change c
JOIN ChangeActionPlan p ON p.changeID = c.changeID
JOIN ChangeActionPlanLine l ON l.actionPlanID = p.actionPlanID
GROUP BY EXTRACT(YEAR FROM changeDate);

--En el año en curso, número total de cambios aprobados en la GCAB
UPDATE StatisticGCABApprovalsCurrentYear SET
currentYear = EXTRACT(YEAR FROM SYSDATE),
numGCABApprovalsCurrentYear = 
 (
  SELECT COUNT(*) FROM Change c
  WHERE
   EXTRACT(YEAR FROM changeDate) = EXTRACT(YEAR FROM SYSDATE) AND
   NOT EXISTS (SELECT * FROM ChangeApprover a WHERE rolID >= 3 AND approved <> 'Y' AND c.changeID = a.changeID)
 );

--En un año concreto, número de cambios que no fueron aprobados y, consecuentemente, no se ejecutaron
DELETE FROM StatisticDisapprovedChangesYear;
INSERT INTO StatisticDisapprovedChangesYear
(
 SELECT EXTRACT(YEAR FROM changeDate), COUNT(*) FROM Change WHERE approvalStatusID > 1 GROUP BY EXTRACT(YEAR FROM changeDate)
);

--Dada una región geográfica concreta, porcentaje de cambios que su ejecución se replanificó
DELETE FROM StatisticReplannedChangesPercentageRegion;
INSERT INTO StatisticReplannedChangesPercentageRegion
 (
  SELECT geographicZoneID, COUNT(*), COUNT(CASE WHEN executionResultID = 3 THEN 1 END)
  FROM Change c
  JOIN ChangeGeographicScope g ON g.changeID = c.changeID
  GROUP BY geographicZoneID
 );

--En los últimos 6 meses, número total de incumplimientos detectados durante las auditorías realizadas
UPDATE StatisticNonCompliancesSixMonths
SET lastMonth = TO_CHAR(SYSDATE, 'YYYYMM'),
month1 = (SELECT COUNT(*) FROM ChangeAuditDetail d JOIN ChangeAudit a ON d.auditID = a.auditID JOIN Change c ON a.changeID = c.changeID WHERE nonCompliance = 'N' AND TO_CHAR(SYSDATE, 'YYYYMM') = TO_CHAR(changeDate, 'YYYYMM')),
month2 = (SELECT COUNT(*) FROM ChangeAuditDetail d JOIN ChangeAudit a ON d.auditID = a.auditID JOIN Change c ON a.changeID = c.changeID WHERE nonCompliance = 'N' AND TO_CHAR(ADD_MONTHS(SYSDATE, -1), 'YYYYMM') = TO_CHAR(changeDate, 'YYYYMM')),
month3 = (SELECT COUNT(*) FROM ChangeAuditDetail d JOIN ChangeAudit a ON d.auditID = a.auditID JOIN Change c ON a.changeID = c.changeID WHERE nonCompliance = 'N' AND TO_CHAR(ADD_MONTHS(SYSDATE, -2), 'YYYYMM') = TO_CHAR(changeDate, 'YYYYMM')),
month4 = (SELECT COUNT(*) FROM ChangeAuditDetail d JOIN ChangeAudit a ON d.auditID = a.auditID JOIN Change c ON a.changeID = c.changeID WHERE nonCompliance = 'N' AND TO_CHAR(ADD_MONTHS(SYSDATE, -3), 'YYYYMM') = TO_CHAR(changeDate, 'YYYYMM')),
month5 = (SELECT COUNT(*) FROM ChangeAuditDetail d JOIN ChangeAudit a ON d.auditID = a.auditID JOIN Change c ON a.changeID = c.changeID WHERE nonCompliance = 'N' AND TO_CHAR(ADD_MONTHS(SYSDATE, -4), 'YYYYMM') = TO_CHAR(changeDate, 'YYYYMM')),
month6 = (SELECT COUNT(*) FROM ChangeAuditDetail d JOIN ChangeAudit a ON d.auditID = a.auditID JOIN Change c ON a.changeID = c.changeID WHERE nonCompliance = 'N' AND TO_CHAR(ADD_MONTHS(SYSDATE, -5), 'YYYYMM') = TO_CHAR(changeDate, 'YYYYMM'));

--En el momento de ejecutar la consulta, número total de planes de acción sobre ejecuciones no correctas que están abiertos
UPDATE StatisticOpenedActionPlans
SET numOpenedActionPlans = (SELECT COUNT(*) FROM ChangeActionPlan WHERE closed IS NULL);

--En un momento dado, responsable técnico con más cambios en curso
DELETE FROM StatisticTopTechnicalManagerChangesInProgress;
INSERT INTO StatisticTopTechnicalManagerChangesInProgress
SELECT CASE WHEN substituteID IS NOT NULL THEN substituteID ELSE usualID END, COUNT(*)
FROM ChangeApprover a JOIN Change c ON a.changeID = c.changeID
WHERE rolID = 1 AND executionResultID = 0 GROUP BY CASE WHEN substituteID IS NOT NULL THEN substituteID ELSE usualID END;


--Teniendo en cuenta todos los datos de la BD, persona concreta con rol de aprobador que ha estado sustituida más veces por indisponibilidad no planificada
DELETE FROM StatisticSubstitutionApprover;
INSERT INTO StatisticSubstitutionApprover
SELECT usualID, COUNT(*)
FROM ChangeApprover a JOIN substitutionReason r ON a.substitutionReasonID = r.substitutionReasonID
WHERE planned = 'N'
GROUP BY usualID;
